import { DisplayValue } from './display-value.js';
import { randomID } from '../../util/helpers.js';
import { Rectangle } from './geometry.js';

export class Box {
  constructor(name, parentEl, options = {}) {
    this.name = name;
    const { tagName = 'div' } = options;
    this.el = document.createElement(tagName);
    this.el.box = this;
    const id = options.id ?? randomID();
    this.el.id = `${parentEl.id}_box_${id}`;
    this.el.classList.add('box');
    if (name) {
      this.el.setAttribute('box-name', name);
    }
    if (parentEl) {
      if (options.prepend) {
        parentEl.prepend(this.el);
      } else {
        parentEl.appendChild(this.el);
      }
    }
    this.boxes = [];
  }

  flex({ center = false } = {}) {
    this.addClass('flex');
    if (center) {
      this.addClass('flex-center');
    }
    return this;
  }

  monospace() {
    this.addClass('monospace');
    return this;
  }

  addClass(className) {
    this.el.classList.add(className);
    return this;
  }

  addBox(name) {
    const box = new Box(name, this.el);
    this.boxes.push(box);
    return box;
  }

  addDisplayValue(value) {
    const box = this.addBox(value.name).flex();
    return new DisplayValue(value, box);
  }

  setInnerHTML(html) {
    this.el.innerHTML = html;
    return this;
  }

  getId() {
    return this.el.id;
  }

  getGeometry() {
    const {
      x, y, width, height,
    } = this.el.getBoundingClientRect();
    return new Rectangle([x, y], [width, height]);
  }

  move(vector) {
    this.el.style.left = `${parseInt(this.el.style.left, 10) + vector[0]}px`;
    this.el.style.top = `${parseInt(this.el.style.top, 10) + vector[1]}px`;
  }
}
